@@ -1,6 +1,6 @@
 /* _NVRM_COPYRIGHT_BEGIN_
  *
- * Copyright 2013-2015 by NVIDIA Corporation.  All rights reserved.  All
+ * Copyright 2013-2019 by NVIDIA Corporation.  All rights reserved.  All
  * information contained herein is proprietary and confidential to NVIDIA
  * Corporation.  Any use, reproduction, or disclosure without the written
  * permission of NVIDIA Corporation is prohibited.
@@ -109,20 +109,43 @@ NV_STATUS nvUvmInterfaceSessionCreate(uvmGpuSessionHandle *session,
 */
 NV_STATUS nvUvmInterfaceSessionDestroy(uvmGpuSessionHandle session);
 
+/*******************************************************************************
+    nvUvmInterfaceDeviceCreate
+
+    Creates a device object under the given session for the GPU with the given
+    UUID. A device handle is returned in the device output parameter.
+
+    Error codes:
+      NV_ERR_GENERIC
+      NV_ERR_NO_MEMORY
+      NV_ERR_INVALID_ARGUMENT
+      NV_ERR_INSUFFICIENT_RESOURCES
+      NV_ERR_OBJECT_NOT_FOUND
+*/
+NV_STATUS nvUvmInterfaceDeviceCreate(uvmGpuSessionHandle session,
+                                     const NvProcessorUuid *gpuUuid,
+                                     uvmGpuDeviceHandle *device);
+
+/*******************************************************************************
+    nvUvmInterfaceDeviceDestroy
+
+    Destroys the device object for the given handle. The handle must have been
+    obtained in a prior call to nvUvmInterfaceDeviceCreate.
+*/
+void nvUvmInterfaceDeviceDestroy(uvmGpuDeviceHandle device);
+
 /*******************************************************************************
     nvUvmInterfaceAddressSpaceCreate
 
     This function creates an address space.
     This virtual address space is created on the GPU specified
-    by the gpuUuid.
-
+    by device.
 
     Error codes:
       NV_ERR_GENERIC
       NV_ERR_NO_MEMORY
 */
-NV_STATUS nvUvmInterfaceAddressSpaceCreate(uvmGpuSessionHandle session,
-                                           const NvProcessorUuid *gpuUuid,
+NV_STATUS nvUvmInterfaceAddressSpaceCreate(uvmGpuDeviceHandle device,
                                            unsigned long long vaBase,
                                            unsigned long long vaSize,
                                            uvmGpuAddressSpaceHandle *vaSpace,
@@ -133,14 +156,13 @@ NV_STATUS nvUvmInterfaceAddressSpaceCreate(uvmGpuSessionHandle session,
 
     This function will dup the given vaspace from the users client to the 
     kernel client was created as an ops session.
-    
+
     By duping the vaspace it is guaranteed that RM will refcount the vaspace object.
 
     Error codes:
       NV_ERR_GENERIC
 */
-NV_STATUS nvUvmInterfaceDupAddressSpace(uvmGpuSessionHandle session,
-                                        const NvProcessorUuid *gpuUuid,
+NV_STATUS nvUvmInterfaceDupAddressSpace(uvmGpuDeviceHandle device,
                                         NvHandle hUserClient,
                                         NvHandle hUserVASpace,
                                         uvmGpuAddressSpaceHandle *vaSpace,
@@ -253,28 +275,27 @@ NV_STATUS nvUvmInterfaceMemoryAllocSys(uvmGpuAddressSpaceHandle vaSpace,
     Obtain the P2P capabilities between the devices with the given gpu address
     space handles.
 
-    This function accepts gpu address space handles or NULL to indicate cpu.
-    At least one gpu address space handle is required. Only cpus connected
-    to a gpu using an npu + nvlink2 or greater are considered to be peers.
+    This function accepts gpu address space handles, both are required.
 
-    If a cpu is included in the query or indirect access is reported, the
-    contents of peerIds are undefined.
+    If indirect access is reported, the contents of peerIds are undefined.
 
     Arguments:
-        vaSpace1[IN]              - first gpu address space handle
-        vaSpace1[IN]              - second gpu address space handle
+        vaSpace1[IN]               - first gpu address space handle
+        vaSpace1[IN]               - second gpu address space handle
         p2pCapsParams members:
-        peerLink[OUT]             - type of the link enabled between the peers
-        peerIds[OUT]              - peer ids between given pair of peers
-        writeSupported[OUT]       - p2p writes between peers are supported
-        readSupported[OUT]        - p2p reads between peers are supported 
-        atomicSupported[OUT]      - p2p atomics between peers are supported
-        directAccess[OUT]         - if a valid peer link is returned, this
-                                    field tells whether there is a direct link
-                                    between the devices or they communicate
-                                    through an intermediate device such as
-                                    a npu. If indirect access is reported link
-                                    must be NVLink2 or greater.
+        peerLink[OUT]              - type of the link enabled between the peers
+        peerIds[OUT]               - peer ids between given pair of peers
+        writeSupported[OUT]        - p2p writes between peers are supported
+        readSupported[OUT]         - p2p reads between peers are supported
+        atomicSupported[OUT]       - p2p atomics between peers are supported
+        totalLinkLineRateMBps[OUT] - Maximum unidirectional bandwidth between
+                                     the peers in megabytes per second
+        directAccess[OUT]          - if a valid peer link is returned, this
+                                     field tells whether there is a direct link
+                                     between the devices or they communicate
+                                     through an intermediate device such as
+                                     a npu. If indirect access is reported link
+                                     must be NVLink2 or greater.
 
     Error codes:
         NV_ERR_INVALID_ARGUMENT
@@ -290,14 +311,14 @@ NV_STATUS nvUvmInterfaceGetP2PCaps(uvmGpuAddressSpaceHandle vaSpace1,
 /*******************************************************************************
     nvUvmInterfaceGetPmaObject
 
-    This function will returns pointer to PMA object for the GPU whose UUID is 
-    passed as an argument. This PMA object handle is required for invoking PMA 
-    for allocate and free pages.
+    This function will return pointer to PMA object for the given GPU. This
+    PMA object handle is required for page allocation.
 
     Arguments:
-        uuidMsb [IN]        - MSB part of the GPU UUID.
-        uuidLsb [IN]        - LSB part of the GPU UUID.
+        vaSpace [IN]        - VA space handle allocated in
+                              nvUvmInterfaceAddressSpaceCreate
         pPma [OUT]          - Pointer to PMA object
+        pPmaPubStats [OUT]  - Pointer to UvmPmaStatistics object
 
     Error codes:
         NV_ERR_NOT_SUPPORTED          - Operation not supported on broken FB
@@ -305,7 +326,9 @@ NV_STATUS nvUvmInterfaceGetP2PCaps(uvmGpuAddressSpaceHandle vaSpace1,
           Unexpected error. We try hard to avoid returning this error
           code,because it is not very informative.
 */
-NV_STATUS nvUvmInterfaceGetPmaObject(const NvProcessorUuid * gpuUUID, void **pPma);
+NV_STATUS nvUvmInterfaceGetPmaObject(uvmGpuAddressSpaceHandle vaSpace,
+                                     void **pPma,
+                                     const UvmPmaStatistics **pPmaPubStats);
 
 // Mirrors pmaEvictPagesCb_t, see its documentation in pma.h.
 typedef NV_STATUS (*uvmPmaEvictPagesCallback)(void *callbackData,
@@ -591,7 +614,7 @@ NV_STATUS nvUvmInterfaceGetGpuInfo(const NvProcessorUuid *gpuUuid, UvmGpuInfo *p
       NV_ERR_GENERIC
       UVM_INVALID_ARGUMENTS
 */
-NV_STATUS nvUvmInterfaceServiceDeviceInterruptsRM(uvmGpuAddressSpaceHandle vaSpace);
+NV_STATUS nvUvmInterfaceServiceDeviceInterruptsRM(uvmGpuDeviceHandle device);
 
 /*******************************************************************************
     nvUvmInterfaceSetPageDirectory
@@ -764,7 +787,8 @@ NV_STATUS nvUvmInterfaceGetFbInfo(uvmGpuAddressSpaceHandle vaSpace,
       NV_ERR_GENERIC
       NV_ERR_INVALID_ARGUMENT
 */
-NV_STATUS nvUvmInterfaceOwnPageFaultIntr(const NvProcessorUuid *gpuUuid, NvBool bOwnInterrupts);
+NV_STATUS nvUvmInterfaceOwnPageFaultIntr(uvmGpuDeviceHandle device, NvBool bOwnInterrupts);
+
 /*******************************************************************************
     nvUvmInterfaceInitFaultInfo
 
@@ -773,7 +797,7 @@ NV_STATUS nvUvmInterfaceOwnPageFaultIntr(const NvProcessorUuid *gpuUuid, NvBool
     faults if the GPU supports it.
 
     Arguments:
-        vaspace[IN]       - Pointer to vaSpace object associated with the gpu
+        device[IN]        - Device handle associated with the gpu
         pFaultInfo[OUT]   - information provided by RM for fault handling
 
     Error codes:
@@ -781,7 +805,7 @@ NV_STATUS nvUvmInterfaceOwnPageFaultIntr(const NvProcessorUuid *gpuUuid, NvBool
       NV_ERR_NO_MEMORY
       NV_ERR_INVALID_ARGUMENT
 */
-NV_STATUS nvUvmInterfaceInitFaultInfo(uvmGpuAddressSpaceHandle vaSpace,
+NV_STATUS nvUvmInterfaceInitFaultInfo(uvmGpuDeviceHandle device,
                                       UvmGpuFaultInfo *pFaultInfo);
 
 /*******************************************************************************
@@ -791,14 +815,14 @@ NV_STATUS nvUvmInterfaceInitFaultInfo(uvmGpuAddressSpaceHandle vaSpace,
     for replayable faults, and frees the shadow buffer for non-replayable faults.
 
     Arguments:
-        vaspace[IN]       - Pointer to vaSpace object associated with the gpu
+        device[IN]        - Device handle associated with the gpu
         pFaultInfo[OUT]   - information provided by RM for fault handling
 
     Error codes:
       NV_ERR_GENERIC
       NV_ERR_INVALID_ARGUMENT
 */
-NV_STATUS nvUvmInterfaceDestroyFaultInfo(uvmGpuAddressSpaceHandle vaSpace,
+NV_STATUS nvUvmInterfaceDestroyFaultInfo(uvmGpuDeviceHandle device,
                                          UvmGpuFaultInfo *pFaultInfo);
 
 /*******************************************************************************
@@ -859,7 +883,8 @@ NV_STATUS nvUvmInterfaceHasPendingNonReplayableFaults(UvmGpuFaultInfo *pFaultInf
       NV_ERR_INVALID_ARGUMENT
 */
 NV_STATUS nvUvmInterfaceGetNonReplayableFaults(UvmGpuFaultInfo *pFaultInfo,
-                                               void *pFaultBuffer, NvU32 *numFaults);
+                                               void *pFaultBuffer,
+                                               NvU32 *numFaults);
 
 /*******************************************************************************
     nvUvmInterfaceInitAccessCntrInfo
@@ -867,15 +892,15 @@ NV_STATUS nvUvmInterfaceGetNonReplayableFaults(UvmGpuFaultInfo *pFaultInfo,
     This function obtains access counter buffer address, size and a few register mappings
  
     Arguments:
-        vaspace[IN]          - Pointer to vaSpace object associated with the gpu
+        device[IN]           - Device handle associated with the gpu
         pAccessCntrInfo[OUT] - Information provided by RM for access counter handling
 
     Error codes:
       NV_ERR_GENERIC
       NV_ERR_INVALID_ARGUMENT
 */
-NV_STATUS nvUvmInterfaceInitAccessCntrInfo(uvmGpuAddressSpaceHandle vaSpace,
-    UvmGpuAccessCntrInfo *pAccessCntrInfo);
+NV_STATUS nvUvmInterfaceInitAccessCntrInfo(uvmGpuDeviceHandle device,
+                                           UvmGpuAccessCntrInfo *pAccessCntrInfo);
 
 /*******************************************************************************
     nvUvmInterfaceDestroyAccessCntrInfo
@@ -883,15 +908,15 @@ NV_STATUS nvUvmInterfaceInitAccessCntrInfo(uvmGpuAddressSpaceHandle vaSpace,
     This function obtains, destroys, unmaps the access counter buffer and clears accessCntrInfo
  
     Arguments:
-        vaspace[IN]         - Pointer to vaSpace object associated with the gpu
+        device[IN]          - Device handle associated with the gpu
         pAccessCntrInfo[IN] - Information provided by RM for access counter handling
 
     Error codes:
       NV_ERR_GENERIC
       NV_ERR_INVALID_ARGUMENT
 */
-NV_STATUS nvUvmInterfaceDestroyAccessCntrInfo(uvmGpuAddressSpaceHandle vaSpace,
-    UvmGpuAccessCntrInfo *pAccessCntrInfo);
+NV_STATUS nvUvmInterfaceDestroyAccessCntrInfo(uvmGpuDeviceHandle device,
+                                              UvmGpuAccessCntrInfo *pAccessCntrInfo);
 
 /*******************************************************************************
     nvUvmInterfaceEnableAccessCntr
@@ -904,7 +929,7 @@ NV_STATUS nvUvmInterfaceDestroyAccessCntrInfo(uvmGpuAddressSpaceHandle vaSpace,
     function before it returns.
 
     Arguments:
-        vaspace[IN]           - Pointer to vaSpace object associated with the gpu
+        device[IN]            - Device handle associated with the gpu
         pAccessCntrInfo[IN]   - Pointer to structure filled out by nvUvmInterfaceInitAccessCntrInfo
         pAccessCntrConfig[IN] - Configuration for access counters
 
@@ -912,9 +937,9 @@ NV_STATUS nvUvmInterfaceDestroyAccessCntrInfo(uvmGpuAddressSpaceHandle vaSpace,
       NV_ERR_GENERIC
       NV_ERR_INVALID_ARGUMENT
 */
-NV_STATUS nvUvmInterfaceEnableAccessCntr(uvmGpuAddressSpaceHandle vaSpace,
-    UvmGpuAccessCntrInfo *pAccessCntrInfo,
-    UvmGpuAccessCntrConfig *pAccessCntrConfig);
+NV_STATUS nvUvmInterfaceEnableAccessCntr(uvmGpuDeviceHandle device,
+                                         UvmGpuAccessCntrInfo *pAccessCntrInfo,
+                                         UvmGpuAccessCntrConfig *pAccessCntrConfig);
 
 /*******************************************************************************
     nvUvmInterfaceDisableAccessCntr
@@ -925,15 +950,15 @@ NV_STATUS nvUvmInterfaceEnableAccessCntr(uvmGpuAddressSpaceHandle vaSpace,
     are disabled by this function before it returns.
 
     Arguments:
-        vaspace[IN]          - Pointer to vaSpace object associated with the gpu
+        device[IN]           - Device handle associated with the gpu
         pAccessCntrInfo[IN]  - Pointer to structure filled out by nvUvmInterfaceInitAccessCntrInfo
 
     Error codes:
       NV_ERR_GENERIC
       NV_ERR_INVALID_ARGUMENT
 */
-NV_STATUS nvUvmInterfaceDisableAccessCntr(uvmGpuAddressSpaceHandle vaSpace,
-    UvmGpuAccessCntrInfo *pAccessCntrInfo);
+NV_STATUS nvUvmInterfaceDisableAccessCntr(uvmGpuDeviceHandle device,
+                                          UvmGpuAccessCntrInfo *pAccessCntrInfo);
 
 //
 // Called by the UVM driver to register operations with RM. Only one set of
