@@ -339,7 +339,7 @@ do {                                                                           \
  */
 #define NvU64_ALIGN32_UNPACK(pDst, pSrc)                                       \
 do {                                                                           \
-    (*pDst) = NvU64_ALIGN32_VAL(pSrc);                                         \
+    (*(pDst)) = NvU64_ALIGN32_VAL(pSrc);                                       \
 } while (NV_FALSE)
 
 /*!
