@@ -1,5 +1,5 @@
 /*******************************************************************************
-    Copyright (c) 2016 NVIDIA Corporation
+    Copyright (c) 2016-2019 NVIDIA Corporation
 
     Permission is hereby granted, free of charge, to any person obtaining a copy
     of this software and associated documentation files (the "Software"), to
@@ -584,7 +584,7 @@ static NV_STATUS uvm_map_external_allocation_on_gpu(uvm_va_range_t *va_range,
 
     // Insert the ext_gpu_map into the VA range immediately since some of the
     // below calls require it to be there.
-    va_range->external.gpu_maps[uvm_gpu_index(mapping_gpu->id)] = ext_gpu_map;
+    va_range->external.gpu_maps[uvm_id_gpu_index(mapping_gpu->id)] = ext_gpu_map;
     uvm_processor_mask_set(&va_range->external.mapped_gpus, mapping_gpu->id);
     ext_gpu_map->gpu = mapping_gpu;
 
@@ -745,7 +745,7 @@ error:
     (void)uvm_tracker_wait_deinit(&tracker);
 
     // Tear down only those mappings we created during this call
-    for_each_gpu_in_mask(mapping_gpu, &mapped_gpus)
+    for_each_va_space_gpu_in_mask(mapping_gpu, va_space, &mapped_gpus)
         uvm_ext_gpu_map_destroy(va_range, mapping_gpu, NULL);
 
     if (uvm_processor_mask_empty(&va_range->external.mapped_gpus)) {
@@ -850,7 +850,7 @@ void uvm_ext_gpu_map_destroy(uvm_va_range_t *va_range, uvm_gpu_t *mapped_gpu, st
         uvm_ext_gpu_map_free_internal(ext_gpu_map);
     }
 
-    va_range->external.gpu_maps[uvm_gpu_index(mapped_gpu->id)] = NULL;
+    va_range->external.gpu_maps[uvm_id_gpu_index(mapped_gpu->id)] = NULL;
     uvm_processor_mask_clear(&va_range->external.mapped_gpus, mapped_gpu->id);
 }
 
@@ -908,7 +908,7 @@ static NV_STATUS uvm_free(uvm_va_space_t *va_space, NvU64 base, NvU64 length)
 {
     uvm_va_range_t *va_range;
     NV_STATUS status = NV_OK;
-    uvm_processor_mask_t retained_mask;
+    uvm_global_processor_mask_t retained_mask;
     LIST_HEAD(deferred_free_list);
 
     if (uvm_api_range_invalid_4k(base, length))
@@ -933,7 +933,7 @@ static NV_STATUS uvm_free(uvm_va_space_t *va_space, NvU64 base, NvU64 length)
     }
 
     if (va_range->type == UVM_VA_RANGE_TYPE_SEMAPHORE_POOL &&
-        uvm_processor_mask_test(&va_range->semaphore_pool.mem->mapped_on, UVM_CPU_ID)) {
+        uvm_global_processor_mask_test(&va_range->semaphore_pool.mem->mapped_on, UVM_GLOBAL_ID_CPU)) {
         // Semaphore pools must be first unmapped from the CPU with munmap to
         // invalidate the vma.
         status = NV_ERR_INVALID_ARGUMENT;
@@ -942,8 +942,8 @@ static NV_STATUS uvm_free(uvm_va_space_t *va_space, NvU64 base, NvU64 length)
 
     if (va_range->type == UVM_VA_RANGE_TYPE_EXTERNAL) {
         // External ranges have deferred free work, so retain their GPUs
-        uvm_processor_mask_copy(&retained_mask, &va_range->external.mapped_gpus);
-        uvm_gpu_retain_mask(&retained_mask);
+        uvm_va_space_global_gpus_in_mask(va_space, &retained_mask, &va_range->external.mapped_gpus);
+        uvm_global_mask_retain(&retained_mask);
     }
 
     uvm_va_range_destroy(va_range, &deferred_free_list);
@@ -954,7 +954,7 @@ out:
     if (!list_empty(&deferred_free_list)) {
         UVM_ASSERT(status == NV_OK);
         uvm_deferred_free_object_list(&deferred_free_list);
-        uvm_gpu_release_mask(&retained_mask);
+        uvm_global_mask_release(&retained_mask);
     }
 
     return status;
