@@ -21,11 +21,6 @@
 
 *******************************************************************************/
 
-/*
- * This file contains common header code that can be used for all variants of
- * the UVM kernel module.
- */
-
 #ifndef _UVM_COMMON_H
 #define _UVM_COMMON_H
 
@@ -47,7 +42,14 @@
 #include "uvmtypes.h"
 #include "uvm_linux.h"
 #include "uvm_utils.h"
-#include "uvm_minimal_init.h"
+
+enum {
+    NVIDIA_UVM_PRIMARY_MINOR_NUMBER = 0,
+    NVIDIA_UVM_TOOLS_MINOR_NUMBER   = 1,
+    // to ensure backward-compatiblity and correct counting, please insert any
+    // new minor devices just above the following field:
+    NVIDIA_UVM_NUM_MINOR_DEVICES
+};
 
 #define UVM_PRINT_FUNC_PREFIX(func, prefix, fmt, ...) \
     func(prefix "%s:%u %s[pid:%d]" fmt,               \
@@ -294,8 +296,6 @@ int nv_status_to_errno(NV_STATUS status);
 unsigned uvm_get_stale_process_id(void);
 unsigned uvm_get_stale_thread_id(void);
 NvBool uvm_user_id_security_check(uid_t euidTarget);
-NV_STATUS uvm_api_stub(void *pParams, struct file *filp);
-NV_STATUS uvm_api_unsupported(void *pParams, struct file *filp);
 
 extern int uvm_enable_builtin_tests;
 
@@ -340,5 +340,8 @@ static inline NvU64 uvm_address_get_canonical_form(NvU64 address)
     __ret;                                \
 })
 
+// Returns whether the input file was opened against the UVM character device
+// file. A NULL input returns false.
+bool uvm_file_is_nvidia_uvm(struct file *filp);
 
 #endif /* _UVM_COMMON_H */
