@@ -1,6 +1,6 @@
 /* _NVRM_COPYRIGHT_BEGIN_
  *
- * Copyright 1999-2018 by NVIDIA Corporation.  All rights reserved.  All
+ * Copyright 1999-2019 by NVIDIA Corporation.  All rights reserved.  All
  * information contained herein is proprietary and confidential to NVIDIA
  * Corporation.  Any use, reproduction, or disclosure without the written
  * permission of NVIDIA Corporation is prohibited.
@@ -15,6 +15,8 @@
 #include "nv-linux.h"
 #include "nv-reg.h"
 
+#include <linux/acpi.h>
+
 #if defined(NV_LINUX_ACPI_EVENTS_SUPPORTED)
 static NV_STATUS   nv_acpi_extract_integer (const union acpi_object *, void *, NvU32, NvU32 *);
 static NV_STATUS   nv_acpi_extract_buffer  (const union acpi_object *, void *, NvU32, NvU32 *);
@@ -62,9 +64,19 @@ static acpi_handle psr_handle = NULL;
 static acpi_handle psr_device_handle = NULL;
 static nv_acpi_t  *psr_nv_acpi_object = NULL;
 
+/* Moved into acpi/video.h in Linux 4.10 */
+#ifndef ACPI_VIDEO_NOTIFY_PROBE
+#define ACPI_VIDEO_NOTIFY_PROBE    0x81
+#endif
+
+/* Added to acpi/video.h in Linux 3.1 */
+#ifndef ACPI_VIDEO_CLASS
+#define ACPI_VIDEO_CLASS    "video"
+#endif
+
 static const struct acpi_driver nv_acpi_driver_template = {
     .name = "NVIDIA ACPI Video Driver",
-    .class = "video",
+    .class = ACPI_VIDEO_CLASS,
     .ids = nv_video_device_ids,
     .ops = {
         .add = nv_acpi_add,
@@ -91,24 +103,32 @@ static int nv_acpi_get_device_handle(nv_state_t *nv, acpi_handle *dev_handle)
 #endif
 }
 
-#if defined(NV_REGISTER_ACPI_NOTIFER_PRESENT)
-
-#ifndef ACPI_VIDEO_NOTIFY_PROBE
-#define ACPI_VIDEO_NOTIFY_PROBE    0x81
-#endif
-
-#ifndef ACPI_VIDEO_CLASS
-#define ACPI_VIDEO_CLASS    "video"
-#endif
-
 static int nv_acpi_notify(struct notifier_block *nb,
                         unsigned long val, void *data)
 {
     struct acpi_bus_event *info = data;
+    nv_stack_t *sp = NULL;
+    nv_linux_state_t *nvl = container_of(nb, nv_linux_state_t, acpi_nb);
+    nv_state_t *nv = NV_STATE_PTR(nvl);
 
     if (!strcmp(info->device_class, ACPI_VIDEO_CLASS)) {
         if (info->type == ACPI_VIDEO_NOTIFY_PROBE) {
-            /* acpi-video should not generate keypresses for this */
+            if (nv_kmem_cache_alloc_stack(&sp) == 0) {
+                /*
+                 * If GPU is in GC6 state, this function wakes up the GPU
+                 * and notifies the client about display hot-plug/unplug.
+                 */
+                rm_handle_display_change(sp, nv);
+                nv_kmem_cache_free_stack(sp);
+            }
+            else
+                nv_printf(NV_DBG_ERRORS,
+                    "NVRM: nv_acpi_notify: failed to allocate stack\n");
+
+            /*
+             * NOTIFY_BAD tells acpi-video to avoid generating
+             * keypresses for this event.
+             */
             return NOTIFY_BAD;
         }
     }
@@ -116,10 +136,16 @@ static int nv_acpi_notify(struct notifier_block *nb,
     return NOTIFY_DONE;
 }
 
-struct notifier_block acpi_nb = {
-    .notifier_call = nv_acpi_notify,
-};
-#endif
+void nv_acpi_register_notifier(nv_linux_state_t *nvl)
+{
+    nvl->acpi_nb.notifier_call = nv_acpi_notify;
+    register_acpi_notifier(&nvl->acpi_nb);
+}
+
+void nv_acpi_unregister_notifier(nv_linux_state_t *nvl)
+{
+    unregister_acpi_notifier(&nvl->acpi_nb);
+}
 
 int nv_acpi_init(void)
 {
@@ -165,10 +191,6 @@ int nv_acpi_init(void)
         nv_acpi_driver = NULL;
     }
 
-#if defined(NV_REGISTER_ACPI_NOTIFER_PRESENT)
-    register_acpi_notifier(&acpi_nb);
-#endif
-
     return status;
 }
 
@@ -179,10 +201,6 @@ int nv_acpi_uninit(void)
     NV_STATUS rmStatus;
     int rc;
 
-#if defined(NV_REGISTER_ACPI_NOTIFER_PRESENT)
-    unregister_acpi_notifier(&acpi_nb);
-#endif
-
     rc = nv_kmem_cache_alloc_stack(&sp);
     if (rc != 0)
     {
@@ -1587,6 +1605,65 @@ static NV_STATUS  nv_acpi_wmmx_method(
     return NV_OK;
 }
 
+NvBool NV_API_CALL nv_acpi_upstreamport_power_resource_method_present(
+    void *upstream_handle
+)
+{
+    struct pci_dev *pdev = upstream_handle;
+    acpi_handle handle = NULL;
+    struct acpi_buffer buf = { ACPI_ALLOCATE_BUFFER, NULL };
+    union acpi_object *object_package, *object_reference;
+    acpi_status status;
+
+#if defined(DEVICE_ACPI_HANDLE)
+    handle = DEVICE_ACPI_HANDLE(&pdev->dev);
+#elif defined (ACPI_HANDLE)
+    handle = ACPI_HANDLE(&pdev->dev);
+#endif
+
+    if (!handle)
+        return NV_FALSE;
+
+    status = acpi_evaluate_object(handle, "_PR3", NULL, &buf);
+    if (ACPI_FAILURE(status))
+    {
+        nv_printf(NV_DBG_INFO,"NVRM: Failed to evaluate _PR3 object\n");
+        return NV_FALSE;
+    }
+
+    if (!buf.pointer)
+    {
+       nv_printf(NV_DBG_INFO, "NVRM: output buffer pointer is null"
+                              " for upstream port's _PR3 method\n");
+       return NV_FALSE;
+    }
+
+    object_package = buf.pointer;
+
+    /*
+     * _PR3 object should be of type package and
+     * it should contain only one reference
+     */
+    if ((object_package->type != ACPI_TYPE_PACKAGE) &&
+        (object_package->package.count != 0x1))
+    {
+        nv_printf(NV_DBG_ERRORS,"NVRM: _PR3 object is not a type 'package'\n");
+        return NV_FALSE;
+    }
+
+    object_reference = object_package->package.elements;
+
+    /* Check for the reference and the actual type of the reference. */
+    if ((object_reference->reference.actual_type != ACPI_TYPE_POWER) &&
+        (object_reference->type !=  ACPI_TYPE_LOCAL_REFERENCE))
+    {
+        nv_printf(NV_DBG_ERRORS,
+                     "NVRM: _PR3 object does not contain POWER Reference\n");
+        return NV_FALSE;
+    }
+    return NV_TRUE;
+}
+
 #else // NV_LINUX_ACPI_EVENTS_SUPPORTED
 
 int nv_acpi_init(void)
@@ -1664,4 +1741,21 @@ NV_STATUS NV_API_CALL nv_acpi_dod_method(
 {
     return NV_ERR_NOT_SUPPORTED;
 }
+
+NvBool NV_API_CALL nv_acpi_upstreamport_power_resource_method_present(
+    void *upstream_handle
+)
+{
+    return NV_FALSE;
+}
+
+void nv_acpi_register_notifier(nv_linux_state_t *nvl)
+{
+    return;
+}
+
+void nv_acpi_unregister_notifier(nv_linux_state_t *nvl)
+{
+    return;
+}
 #endif
